@@ -1,26 +1,29 @@
 import datetime
 import re
-from datetime import date, time, timedelta
-from pandas import isna, to_datetime, Timedelta, Timestamp, to_timedelta
+from collections import UserString
+
+import numpy as np
+import pandas as pd
 from pandas.tseries.frequencies import to_offset
-from numpy import ndarray, NaN
 
 
 def to_bool(x):
     if isinstance(x, bool):
         return x
-    else:
-        if x == 'YES':
+    elif isinstance(x, str):
+        if x.upper() == 'YES':
             return True
-        elif x == 'NO':
+        elif x.upper() == 'NO':
             return False
         else:
-            raise NotImplemented('x not a bool: {}'.format(x))
+            raise NotImplemented(f'x not a "YES" or "NO" but "{x}"')
+    else:
+        raise NotImplemented(f'x not a bool: "{x}" | type={type(x)}')
 
 
 def infer_type(x):
     """
-    infer generic type of inp-file-string
+    Infer generic type of inp-file-string
 
     Args:
         x (str | list[str]):
@@ -28,32 +31,52 @@ def infer_type(x):
     Returns:
         object: object depending on string
     """
-    if isinstance(x, (list, ndarray)):
+    if isinstance(x, (list, np.ndarray)):
         return [infer_type(i) for i in x]
     elif not isinstance(x, str):
         return x
-    elif x == 'YES':
+    elif x.upper() == 'YES':
         return True
-    elif x == 'NO':
+    elif x.upper() == 'NO':
         return False
-    elif x == 'NONE':
+    elif x.upper() == 'NONE':
         return None
     elif x.replace('-', '').isdecimal():
         return int(x)
     elif ('.' in x) and (x.lower().replace('.', '').replace('-', '').replace('e', '').isdecimal()):
         return float(x)
     elif x.count('/') == 2:
-        return to_datetime(x, format='%m/%d/%Y').date()
+        return datetime.datetime.strptime(x, '%m/%d/%Y').date()
     # elif x.count('/') == 1:
     #     return to_datetime(x, format='%m/%d').date()
     elif (x.count(':') == 2) and (len(x) == 8):
-        return to_datetime(x, format='%H:%M:%S').time()
+        return datetime.datetime.strptime(x, '%H:%M:%S').time()
     elif (x.count(':') == 1) and (len(x) == 5):
-        return to_datetime(x, format='%H:%M').time()
+        return datetime.datetime.strptime(x, '%H:%M').time()
     else:
         return x
 
 
+def infer_offset_elevation(x):
+    """
+    Convert string to float.
+
+    Used for link offset parameter.
+    "*" if the option in SWMM is set to "offset measured as elevation" and no offset for this link is set.
+    Otherwise, the input is always a float.
+
+    Args:
+        x (str | float): input from object creation
+
+    Returns:
+        float | str: converted output
+    """
+    if isinstance(x, str) and x == '*':
+        return x
+    else:
+        return float(x)
+
+
 def str_to_datetime(date=None, time=None, str_only=False):
     if date:
         if '-' in date:
@@ -109,10 +132,9 @@ def str_to_datetime(date=None, time=None, str_only=False):
         time_format = ''
 
     if str_only:
-        return date + ' ' + time
+        return f'{date} {time}'
     else:
-        return to_datetime(date + ' ' + time,
-                           format=month_format + date_format2 + ' ' + time_format)
+        return datetime.datetime.strptime(f'{date} {time}', f'{month_format}{date_format2} {time_format}')
 
 
 def datetime_to_str(dt):
@@ -130,16 +152,15 @@ def datetime_to_str(dt):
     elif isinstance(dt, str):
         return dt
     elif isinstance(dt, datetime.datetime):
-        return dt.strftime(format='%m/%d/%Y %H:%M:%S')
-
+        return dt.strftime('%m/%d/%Y %H:%M:%S')
 
 
 def time2delta(t):
-    return Timedelta(hours=t.hour, minutes=t.minute, seconds=t.second)
+    return datetime.timedelta(hours=t.hour, minutes=t.minute, seconds=t.second)
 
 
 def delta2time(d):
-    return Timestamp(2000, 1, 1, d.components.hours, d.components.minutes, d.components.seconds).time()
+    return datetime.datetime(2000, 1, 1, d.components.hours, d.components.minutes, d.components.seconds).time()
 
 
 def delta2offset(d):
@@ -147,7 +168,7 @@ def delta2offset(d):
 
 
 def offset2delta(o):
-    return to_timedelta(o)
+    return pd.to_timedelta(o)
 
 
 def delta2str(d):
@@ -166,7 +187,7 @@ def delta2str(d):
 
 def type2str(x):
     """
-    convert any type to a string
+    Convert any type to a string
 
     Args:
         x (any):
@@ -187,22 +208,20 @@ def type2str(x):
     elif isinstance(x, int):
         return str(x)
     elif isinstance(x, float):
-        if isna(x):
+        if pd.isna(x):
             return ''
         if x == 0.0:
             return '0'
         return '{:0.7G}'.format(x)  # .rstrip('0').rstrip('.')
-    elif isinstance(x, date):
-        return x.strftime(format='%m/%d/%Y')
-    elif isinstance(x, time):
-        return x.strftime(format='%H:%M:%S')
-    elif isinstance(x, (Timedelta, timedelta)):
+    elif isinstance(x, datetime.date):
+        return x.strftime('%m/%d/%Y')
+    elif isinstance(x, datetime.time):
+        return x.strftime('%H:%M:%S')
+    elif isinstance(x, (pd.Timedelta, datetime.timedelta)):
         return delta2str(x)
     else:
         return str(x)
 
-import numpy as np
-
 
 def is_equal(x1, x2, precision=3):
     if isinstance(x1, float) and np.isnan(x1) and isinstance(x2, float) and np.isnan(x2):
@@ -216,20 +235,16 @@ def is_equal(x1, x2, precision=3):
 
 
 def convert_string(x) -> str:
-    if isna(x):
+    if pd.isna(x):
         return x
     x = str(x)
-    if ' ' in x:
-        return f'"{x}"'
-    else:
-        s = x.strip('"')
-        if s == '':
-            return NaN
-        else:
-            return s
+    s = x.strip(' "')
+    if s == '':
+        return np.NaN
+    return s
 
 
-GIS_FLOAT_FORMAT = '0.03f'
+GIS_FLOAT_FORMAT = '0.3f'
 
 _SECTION_PATTERN = re.compile(r'^[ \t]*([^;\n]+)[ \t]*;?[^\n]*$', flags=re.M)
 
@@ -249,3 +264,10 @@ def txt_to_lines(content):
     """
     for line in _SECTION_PATTERN.finditer(content):
         yield line.group().split()
+
+
+class CaseInsensitiveString(UserString):
+    pass
+
+
+cistr = CaseInsensitiveString
