@@ -1,3 +1,5 @@
+from pathlib import Path
+
 import os
 import re
 import warnings
@@ -5,16 +7,13 @@ import warnings
 from .helpers import (section_to_string, CustomDict, convert_section, InpSection,
                       InpSectionGeneric, SECTION_ORDER_DEFAULT, check_order, SECTIONS_ORDER_MP, head_to_str,
                       iter_section_lines, SwmmInputWarning, BaseSectionObject, )
-from .._read_txt import detect_encoding, read_txt_file
+from .._io_helpers import get_default_encoding, read_txt_file
 from .section_types import SECTION_TYPES
 from .section_labels import *
 from .sections import *
 from .sections.subcatch import INFILTRATION_DICT
 
 
-DEFAULT_ENCODING = 'utf-8'
-
-
 class SwmmInput(CustomDict):
     """
     Overall class for an input file.
@@ -24,17 +23,17 @@ class SwmmInput(CustomDict):
     Just used for the copy function and to identify ``.inp``-file data.
     """
 
-    def __init__(self, *args, custom_section_handler=None, encoding=DEFAULT_ENCODING, force_ignore_case=False, **kwargs):
+    def __init__(self, *args, custom_section_handler=None, encoding='', force_ignore_case=False, **kwargs):
         """
 
         Args:
             *args: only for creating the inp-object as a dict.
             custom_section_handler:
-            encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
+            encoding (str): Encoding of the text-file (None -> auto-detect encoding ... takes a few seconds | '' -> use default = 'utf-8')
             **kwargs: only for creating the inp-object as a dict.
         """
         filename = None
-        if len(args) == 1 and isinstance(args[0], str) and os.path.isfile(args[0]):
+        if len(args) == 1 and isinstance(args[0], (str, Path)) and os.path.isfile(args[0]):
             # argument is an inp-file.
             super().__init__()
             filename = args[0]
@@ -42,7 +41,7 @@ class SwmmInput(CustomDict):
             super().__init__(*args, **kwargs)
         self._converter = SECTION_TYPES.copy()
 
-        self._default_encoding = encoding
+        self._default_encoding = encoding  # ''->default | None->autodetect | str->custom
 
         if custom_section_handler is not None:
             self._converter.update(custom_section_handler)
@@ -51,11 +50,14 @@ class SwmmInput(CustomDict):
         self._original_section_order = SECTIONS_ORDER_MP
 
         if filename is not None:
-            self._init_from_file(filename, force_ignore_case=force_ignore_case, encoding=encoding)
+            self._init_from_file(filename, force_ignore_case=force_ignore_case)
 
     def copy(self):
         """Copy inp-data."""
-        new = type(self)()
+        new = type(self)()  # type: SwmmInput
+        new._converter = self._converter
+        new._default_encoding = self._default_encoding
+        new._original_section_order = self._original_section_order
         for key in self:
             if hasattr(self._data[key], 'copy'):
                 new._data[key] = self._data[key].copy()
@@ -71,7 +73,7 @@ class SwmmInput(CustomDict):
             else:
                 if isinstance(self._data[sec], str):
                     if isinstance(d._data[sec], str):
-                        self._data[sec] += '\n' + d._data[sec]
+                        self._data[sec] += f'\n{d._data[sec]}'
                     else:
                         self[sec].update(d[sec])
                         # warnings.warn(f'Updating of string section in INP-file not implemented! Skip Section {sec}')
@@ -89,7 +91,7 @@ class SwmmInput(CustomDict):
                                re.split(r'\[\w+\]', txt)[1:]):
             head = head.upper()
             if head in self._data:
-                self._data[head] += '\n' + lines.strip()
+                self._data[head] += f'\n{lines.strip()}'
             else:
                 self._data[head] = lines.strip()
                 self._original_section_order.append(head)
@@ -103,7 +105,7 @@ class SwmmInput(CustomDict):
 
         self.set_default_infiltration_from_options()
 
-    def _init_from_file(self, filename, force_ignore_case=False, encoding=DEFAULT_ENCODING):
+    def _init_from_file(self, filename, force_ignore_case=False):
         """
         Read ``.inp``-file and convert the sections in pythonic objects.
 
@@ -113,15 +115,11 @@ class SwmmInput(CustomDict):
             filename (str): path/filename to .inp file
             force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
                                         all text/labels will be set to uppercase
-            encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
         """
-        if encoding is None:
-            encoding = detect_encoding(filename)
-
-        self._default_encoding = encoding
+        self._default_encoding = get_default_encoding(self._default_encoding, filename)
 
         if os.path.isfile(filename) or filename.endswith('.inp'):
-            txt = read_txt_file(filename, encoding=encoding)
+            txt = read_txt_file(filename, encoding=self._default_encoding)
 
         else:
             warnings.warn('Reading a string with SwmmInput.read_file is deprecated. Use SwmmInput.read_text instead.', DeprecationWarning)
@@ -130,24 +128,24 @@ class SwmmInput(CustomDict):
         self._init_from_str(txt, force_ignore_case=force_ignore_case)
 
     @classmethod
-    def read_file(cls, filename, custom_converter=None, force_ignore_case=False, encoding=DEFAULT_ENCODING):
+    def read_file(cls, filename, custom_converter=None, force_ignore_case=False, encoding=''):
         """
         Read ``.inp``-file and convert the sections in pythonic objects.
 
         The sections will be converted when used.
 
         Args:
-            filename (str): path/filename to .inp file
+            filename (str | Path): path/filename to .inp file
             custom_converter (dict): dictionary of {section: converter/section_type} Default: :py:const:`SECTION_TYPES`
             force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
                                         all text/labels will be set to uppercase
-            encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
+            encoding (str): Encoding of the text-file (None -> auto-detect encoding ... takes a few seconds | '' -> use default = 'utf-8')
 
         Returns:
             SwmmInput: dict-like data of the sections in the ``.inp``-file
         """
-        inp = cls(custom_section_handler=custom_converter)
-        inp._init_from_file(filename, force_ignore_case, encoding)
+        inp = cls(custom_section_handler=custom_converter, encoding=encoding)
+        inp._init_from_file(filename, force_ignore_case)
         return inp
 
     @classmethod
@@ -225,8 +223,8 @@ class SwmmInput(CustomDict):
 
         Args:
             infiltration_class: One of
-                :class:`~swmm_api.input_file.sections.InfiltrationCurve`,
-                :class:`~swmm_api.input_file.sections.NumberInfiltrationGreenAmpt`,
+                :class:`~swmm_api.input_file.sections.InfiltrationCurveNumber`,
+                :class:`~swmm_api.input_file.sections.InfiltrationGreenAmpt`,
                 :class:`~swmm_api.input_file.sections.InfiltrationHorton`
         """
         self._converter[INFILTRATION] = infiltration_class
@@ -280,7 +278,7 @@ class SwmmInput(CustomDict):
         Args:
             filename (str): path/filename of created ``.inp``-file
             fast (bool): don't use any formatting else format as table
-            encoding (str): define encoding for resulting inp-file
+            encoding (str): define encoding for resulting inp-file. Default is same as read inp file or package default ('utf-8')
             custom_sections_order (list[str]): list of section names to preset the order of the section in the created
                 inp-file | default: order of the read inp-file + default order of the SWMM GUI
             sort_objects_alphabetical (bool): if objects in a section should be sorted alphabetical |
@@ -290,10 +288,9 @@ class SwmmInput(CustomDict):
         """
 
         if encoding is None:
-            encoding = self._default_encoding
-
-        if encoding is None:
-            encoding = DEFAULT_ENCODING
+            encoding = self._default_encoding   # None->autodetect | str->custom
+            if encoding == '':
+                encoding = get_default_encoding(encoding)
 
         with open(filename, 'w', encoding=encoding) as f:
             for head in self._get_section_headers(custom_sections_order):
@@ -345,7 +342,7 @@ class SwmmInput(CustomDict):
 
     def add_new_section(self, section):
         """
-        add new section to the inp-data
+        Add new section to the inp-data.
 
         Args:
             section (InpSection, InpSectionGeneric):
@@ -360,7 +357,7 @@ class SwmmInput(CustomDict):
 
     def add_obj(self, obj):
         """
-        add object to respective section
+        Add object to respective section.
 
         Args:
             obj (BaseSectionObject):new object
@@ -370,7 +367,7 @@ class SwmmInput(CustomDict):
 
     def add_multiple(self, *items):
         """
-        add multiple objects to respective sections
+        Add multiple objects to respective sections.
 
         Args:
             *items (BaseSectionObject): new objects
@@ -942,24 +939,26 @@ class SwmmInput(CustomDict):
             return self[LID_USAGE]
 
 
-def read_inp_file(filename, custom_converter=None, force_ignore_case=False, encoding=DEFAULT_ENCODING):
-    """
-    Read ``.inp``-file and convert the sections in pythonic objects.
+read_inp_file = SwmmInput.read_file
 
-    The sections will be converted when used.
-
-    Args:
-        filename (str): path/filename to .inp file
-        custom_converter (dict): dictionary of {section: converter/section_type} Default: :py:const:`SECTION_TYPES`
-        force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
-                                    all text/labels will be set to uppercase
-        encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
-
-    Returns:
-        SwmmInput: dict-like data of the sections in the ``.inp``-file
-
-    See Also:
-        :meth:`SwmmInput.read_file` : Equal functionality.
-    """
-    return SwmmInput.read_file(filename, custom_converter=custom_converter, force_ignore_case=force_ignore_case,
-                               encoding=encoding)
+# def read_inp_file(filename, custom_converter=None, force_ignore_case=False, encoding=DEFAULT_ENCODING):
+#     """
+#     Read ``.inp``-file and convert the sections in pythonic objects.
+#
+#     The sections will be converted when used.
+#
+#     Args:
+#         filename (str): path/filename to .inp file
+#         custom_converter (dict): dictionary of {section: converter/section_type} Default: :py:const:`SECTION_TYPES`
+#         force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
+#                                     all text/labels will be set to uppercase
+#         encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
+#
+#     Returns:
+#         SwmmInput: dict-like data of the sections in the ``.inp``-file
+#
+#     See Also:
+#         :meth:`SwmmInput.read_file` : Equal functionality.
+#     """
+#     return SwmmInput.read_file(filename, custom_converter=custom_converter, force_ignore_case=force_ignore_case,
+#                                encoding=encoding)
