@@ -1,10 +1,12 @@
 import inspect
 import time
 
+import pandas as pd
 from numpy import NaN
-from pandas import MultiIndex
+# from pandas import MultiIndex
 from tqdm.auto import tqdm
 
+from .reduce_unneeded import remove_empty_sections
 from .geo import complete_vertices, simplify_vertices, reduce_vertices
 from .tags import get_node_tags, get_link_tags, get_subcatchment_tags, TAG_COL_NAME
 from ..inp import SwmmInput
@@ -90,9 +92,9 @@ def convert_inp_to_geo_package(inp_fn, gpkg_fn=None, driver='GPKG', label_sep='.
     write_geo_package(inp, gpkg_fn, driver=driver, label_sep=label_sep, crs=crs)
 
 
-def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:32633"):
+def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:32633", simplify_link_min_length=None):
     """
-    write the inp file data to a GIS database
+    Write the inp file data to a GIS database.
 
     Args:
         inp (SwmmInput): inp file data
@@ -103,9 +105,12 @@ def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:3263
         crs (str): Coordinate Reference System of the geometry objects.
                     Can be anything accepted by pyproj.CRS.from_user_input(),
                     such as an authority string (eg “EPSG:4326”) or a WKT string.
+        simplify_link_min_length (float): simplify links with a minimum length of ... (default: all)
     """
     from geopandas import GeoDataFrame
 
+    remove_empty_sections(inp)
+
     todo_sections = NODE_SECTIONS + LINK_SECTIONS + [SUBCATCHMENTS]
     print(*todo_sections, sep=' | ')
 
@@ -130,7 +135,8 @@ def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:3263
 
                 df = df.join(inp[COORDINATES].geo_series).join(nodes_tags)
 
-                GeoDataFrame(df).to_file(gpkg_fn, driver=driver, layer=sec)
+                if not df.empty:
+                    GeoDataFrame(df).to_file(gpkg_fn, driver=driver, layer=sec)
                 print(f'{f"{time.perf_counter() - t0:0.1f}s":^{len(sec)}s}', end=' | ')
             else:
                 print(f'{f"-":^{len(sec)}s}', end=' | ')
@@ -139,7 +145,7 @@ def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:3263
         # ---------------------------------
         links_tags = get_link_tags(inp)
         complete_vertices(inp)
-        simplify_vertices(inp)
+        simplify_vertices(inp, min_length=simplify_link_min_length)
         for sec in LINK_SECTIONS:
             if sec in inp:
                 df = inp[sec].frame.rename(columns=lambda c: f'{sec}{label_sep}{c}').join(
@@ -153,7 +159,8 @@ def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:3263
 
                 df = df.join(inp[VERTICES].geo_series).join(links_tags)
 
-                GeoDataFrame(df).to_file(gpkg_fn, driver=driver, layer=sec)
+                if not df.empty:
+                    GeoDataFrame(df).to_file(gpkg_fn, driver=driver, layer=sec)
 
                 print(f'{f"{time.perf_counter() - t0:0.1f}s":^{len(sec)}s}', end=' | ')
             else:
@@ -169,7 +176,8 @@ def write_geo_package(inp, gpkg_fn, driver='GPKG', label_sep='.', crs="EPSG:3263
             inp[INFILTRATION].frame.rename(columns=lambda c: f'{INFILTRATION}{label_sep}{c}')).join(
             inp[POLYGONS].geo_series).join(get_subcatchment_tags(inp))
 
-        GeoDataFrame(df).to_file(gpkg_fn, driver=driver, layer=SUBCATCHMENTS)
+        if not df.empty:
+            GeoDataFrame(df).to_file(gpkg_fn, driver=driver, layer=SUBCATCHMENTS)
 
         print(f'{f"{time.perf_counter() - t0:0.1f}s":^{len(SUBCATCHMENTS)}s}')
 
@@ -254,7 +262,8 @@ def links_geo_data_frame(inp, label_sep='.'):
             if res is None:
                 res = df
             else:
-                res = res.append(df)
+                res = pd.concat([res, df], axis=0)
+                # res = res.append(df)
     return GeoDataFrame(res)
 
 
@@ -292,7 +301,8 @@ def nodes_geo_data_frame(inp, label_sep='.'):
             if res is None:
                 res = df
             else:
-                res = res.append(df)
+                res = pd.concat([res, df], axis=0)
+                # res = res.append(df)
 
     return GeoDataFrame(res)
 
@@ -331,7 +341,7 @@ def gpkg_to_swmm(fn, label_sep='.', infiltration_class=None, custom_section_hand
             cols = gdf.columns[gdf.columns.str.startswith(sub_sec)]
             if not cols.empty:
                 gdf_sub = gdf[cols].copy()
-                gdf_sub.columns = MultiIndex.from_tuples([col.split(label_sep) for col in gdf_sub.columns])
+                gdf_sub.columns = pd.MultiIndex.from_tuples([col.split(label_sep) for col in gdf_sub.columns])
                 cols_order = gdf_sub.columns.droplevel(1)
                 gdf_sub = gdf_sub.stack(1)[cols_order]
                 inp[sub_sec].update(section_dict[sub_sec].create_section(gdf_sub.reset_index().values))
