@@ -1,3 +1,4 @@
+import os
 from statistics import mean
 
 import pandas as pd
@@ -8,7 +9,8 @@ from .tags import get_subcatchment_tags, get_node_tags
 from ..inp import SwmmInput
 from ..section_labels import *
 from ..section_lists import LINK_SECTIONS, NODE_SECTIONS
-from ..sections.link import _Link
+from ..sections import TimeseriesFile
+from ..sections.link import _Link, Conduit, Weir, Outlet, Orifice
 from ..sections.link_component import CrossSection
 
 """
@@ -60,8 +62,8 @@ def calc_slope(inp: SwmmInput, conduit):
         float: slop of the link
     """
     nodes = nodes_dict(inp)
-    return (nodes[conduit.FromNode].Elevation + conduit.InOffset - (
-            nodes[conduit.ToNode].Elevation + conduit.OutOffset)) / conduit.Length
+    return (nodes[conduit.from_node].elevation + conduit.offset_upstream - (
+            nodes[conduit.to_node].elevation + conduit.offset_downstream)) / conduit.length
 
 
 def conduit_slopes(inp: SwmmInput):
@@ -74,9 +76,9 @@ def conduit_slopes(inp: SwmmInput):
     Returns:
         pandas.Series: slopes
     """
-    slopes = dict()
+    slopes = {}
     for conduit in inp.CONDUITS.values():
-        slopes[conduit.Name] = calc_slope(inp, conduit)
+        slopes[conduit.name] = calc_slope(inp, conduit)
     return pd.Series(slopes)
 
 
@@ -90,9 +92,9 @@ def _rel_diff(a, b):
 
 def _rel_slope_diff(inp: SwmmInput, l0, l1):
     nodes = nodes_dict(inp)
-    slope_res = (nodes[l0.FromNode].Elevation + l0.InOffset
-                 - (nodes[l1.ToNode].Elevation + l1.OutOffset)
-                 ) / (l0.Length + l1.Length)
+    slope_res = (nodes[l0.from_node].elevation + l0.offset_upstream
+                 - (nodes[l1.to_node].elevation + l1.offset_downstream)
+                 ) / (l0.length + l1.length)
     return _rel_diff(calc_slope(inp, l0), slope_res)
 
 
@@ -116,25 +118,25 @@ def conduits_are_equal(inp: SwmmInput, link0, link1, diff_roughness=0.1, diff_sl
 
     # Roughness values match within a specified percent tolerance
     if diff_roughness is not None:
-        all_checks_out &= _rel_diff(link0.Roughness, link1.Roughness) < diff_roughness
+        all_checks_out &= _rel_diff(link0.roughness, link1.roughness) < diff_roughness
 
-    xs0 = inp[XSECTIONS][link0.Name]  # type: CrossSection
-    xs1 = inp[XSECTIONS][link1.Name]  # type: CrossSection
+    xs0 = inp[XSECTIONS][link0.name]  # type: CrossSection
+    xs1 = inp[XSECTIONS][link1.name]  # type: CrossSection
 
     # Diameter values match within a specified percent tolerance (1 %)
     if diff_height is not None:
-        all_checks_out &= _rel_diff(xs0.Geom1, xs1.Geom1) < diff_height
+        all_checks_out &= _rel_diff(xs0.height, xs1.height) < diff_height
 
     # Cross-section shapes must match exactly
-    all_checks_out &= xs0.Shape == xs1.Shape
+    all_checks_out &= xs0.shape == xs1.shape
 
     # Shape curves must match exactly
-    if xs0.Shape == CrossSection.SHAPES.CUSTOM:
-        all_checks_out &= xs0.Curve == xs1.Curve
+    if xs0.shape == CrossSection.SHAPES.CUSTOM:
+        all_checks_out &= xs0.curve_name == xs1.curve_name
 
     # Transects must match exactly
-    elif xs0.Shape == CrossSection.SHAPES.IRREGULAR:
-        all_checks_out &= xs0.Tsect == xs1.Tsect
+    elif xs0.shape == CrossSection.SHAPES.IRREGULAR:
+        all_checks_out &= xs0.transect == xs1.transect
 
     # Slope values match within a specified tolerance
     if diff_slope is not None:
@@ -176,7 +178,7 @@ def increase_max_node_depth(inp, node_label):
     node = nodes_dict(inp)[node_label]
     max_height = node.MaxDepth
     for link in previous_ + next_:
-        max_height = max((max_height, inp[XSECTIONS][link.Name].Geom1))
+        max_height = max((max_height, inp[XSECTIONS][link.name].height))
     print(f'MaxDepth increased for node "{node_label}" from {node.MaxDepth} to {max_height}')
     node.MaxDepth = max_height
 
@@ -241,7 +243,7 @@ def nodes_data_frame(inp, label_sep='.'):
         df = inp[s].frame.rename(columns=lambda c: f'{label_sep}{c}')
 
         if s == STORAGE:
-            df[f'{STORAGE}{label_sep}Curve'] = df[f'{STORAGE}{label_sep}Curve'].astype(str)
+            df[f'{STORAGE}{label_sep}data'] = df[f'{STORAGE}{label_sep}data'].astype(str)
 
         for sub_sec in [DWF, INFLOWS]:
             if sub_sec in inp:
@@ -268,3 +270,31 @@ def delete_sections(inp, section_list):
     for s in section_list:
         if s in inp:
             del inp[s]
+
+
+def set_absolute_file_paths(inp, path_data_base):
+    """
+
+    Args:
+        inp (SwmmInput):
+        path_data_base:
+
+    Returns:
+
+    """
+    if FILES in inp:
+        pass  # ?
+
+    if RAINGAGES in inp:
+        for rg in inp.RAINGAGES.values():
+            if isinstance(rg.filename, str):
+                rg.filename = os.path.join(path_data_base, rg.filename)
+
+    if TEMPERATURE in inp:
+        if 'FILE' in inp.TEMPERATURE:
+            inp.TEMPERATURE['FILE'] = os.path.join(path_data_base, inp.TEMPERATURE['FILE'])
+
+    if TIMESERIES in inp:
+        for ts in inp.TIMESERIES.values():
+            if isinstance(ts, TimeseriesFile):
+                ts.filename = os.path.join(path_data_base, ts.filename)
