@@ -1,13 +1,17 @@
 from numpy import NaN
 
 from ._identifiers import IDENTIFIERS
+from .._type_converter import to_bool
 from ..helpers import BaseSectionObject
 from ..section_labels import LID_USAGE, LID_CONTROLS
 
 
 class LIDControl(BaseSectionObject):
     """
-    Section: [**LID_CONTROLS**]
+    Low impact development control information.
+
+    Section:
+        [LID_CONTROLS]
 
     Purpose:
         Defines scale-independent LID controls that can be deployed within subcatchments.
@@ -22,14 +26,6 @@ class LIDControl(BaseSectionObject):
             Name DRAIN Coeff Expon Offset Delay
             Name DRAINMAT Thick Vratio Rough
 
-    Args:
-        Name (str):
-            name assigned to LID process.
-        kind (str):
-            BC for bio-retention cell; RG for rain garden; GR for green roof; IT for
-                infiltration trench; PP for permeable pavement; RB for rain barrel; RD for
-                rooftop disconnection; VS for vegetative swale.
-
     Examples:
         ::
 
@@ -86,8 +82,24 @@ class LIDControl(BaseSectionObject):
 
         The actual dimensions of an LID control are provided in the [LID_USAGE] section when it is
         placed in a particular subcatchment.
+
+    Attributes:
+        name (str): name assigned to LID process.
+        lid_kind (str): One of :attr:`LIDControl.LID_TYPES`.
+
+            - ``BC`` for bio-retention cell
+            - ``RG`` for rain garden; GR for green roof
+            - ``IT`` for infiltration trench
+            - ``PP`` for permeable pavement
+            - ``RB`` for rain barrel
+            - ``RD`` for rooftop disconnection
+            - ``VS`` for vegetative swale.
+
+        layer_dict (dict[str, LIDControl.LAYER_TYPES.Surface | LIDControl.LAYER_TYPES.Soil]): Sict of used layers in control.
+
+        LID_TYPES: Enum-like for the attribute :attr:`LIDControl.lid_kind` with following members -> {``BC`` | ``RG`` | ``IT`` | ``PP`` | ``RB`` | ``RD`` | ``VS``}
     """
-    _identifier = [IDENTIFIERS.Name, 'lid_kind']
+    _identifier = (IDENTIFIERS.name, 'lid_kind')
     _section_label = LID_CONTROLS
     _table_inp_export = False
 
@@ -103,13 +115,27 @@ class LIDControl(BaseSectionObject):
 
         _possible = [BC, RG, GR, IT, PP, RB, RD, VS]
 
-    def __init__(self, Name, lid_kind, layer_dict=None):
-        self.Name = str(Name)
+    def __init__(self, name, lid_kind, layer_dict=None):
+        """
+        Low impact development control information.
+
+        Args:
+            name (str): Name assigned to LID process.
+            lid_kind (str): One of :attr:`LIDControl.LID_TYPES`.
+
+                - ``BC`` for bio-retention cell
+                - ``RG`` for rain garden; GR for green roof
+                - ``IT`` for infiltration trench
+                - ``PP`` for permeable pavement
+                - ``RB`` for rain barrel
+                - ``RD`` for rooftop disconnection
+                - ``VS`` for vegetative swale.
+
+            layer_dict (dict[str, LIDControl.LAYER_TYPES.Surface]): dict of used layers in control.
+        """
+        self.name = str(name)
         self.lid_kind = lid_kind.upper()  # one of LID_TYPES
-        if layer_dict is None:
-            self.layer_dict = dict()
-        else:
-            self.layer_dict = layer_dict
+        self.layer_dict = {} if layer_dict is None else layer_dict
 
     @classmethod
     def _convert_lines(cls, multi_line_args):
@@ -121,7 +147,7 @@ class LIDControl(BaseSectionObject):
                 if last is not None:
                     yield last
                 last = cls(name, lid_kind=line[0].upper())
-            elif name == last.Name:
+            elif name == last.name:
                 surface_kind = line.pop(0).upper()  # one of SURFACE_TYPES
                 last.layer_dict[surface_kind] = cls.LAYER_TYPES._dict[surface_kind](*line)
         yield last
@@ -212,7 +238,7 @@ class LIDControl(BaseSectionObject):
         class Storage(BaseSectionObject):
             _LABEL = 'STORAGE'
 
-            def __init__(self, Height, Vratio, Seepage, Vclog):
+            def __init__(self, Height, Vratio, Seepage, Vclog, Covrd=True):
                 """
                 Args:
                     Height (float): thickness of the storage layer or height of a rain barrel (inches or mm).
@@ -223,6 +249,7 @@ class LIDControl(BaseSectionObject):
                         floor or liner below the layer then use a value of 0.
                     Vclog (int): number of storage layer void volumes of runoff treated it takes to
                         completely clog the layer. Use a value of 0 to ignore clogging.
+                    Covrd (bool): YES (the default) if a rain barrel is covered, NO if it is not.
 
                 Remarks:
                     Values for Vratio, Seepage, and Vclog are ignored for rain barrels.
@@ -231,11 +258,12 @@ class LIDControl(BaseSectionObject):
                 self.Vratio = float(Vratio)
                 self.Seepage = float(Seepage)
                 self.Vclog = int(Vclog)
+                self.Covrd = to_bool(Covrd)
 
         class Drain(BaseSectionObject):
             _LABEL = 'DRAIN'
 
-            def __init__(self, Coeff, Expon, Offset, Delay, open_level=NaN, close_level=NaN):
+            def __init__(self, Coeff, Expon, Offset, Delay, open_level=NaN, close_level=NaN, Qcurve=NaN):
                 """
                 Args:
                     Coeff (float): coefficient C that determines the rate of flow through the drain as a
@@ -251,13 +279,20 @@ class LIDControl(BaseSectionObject):
                         begins). A value of 0 signifies that the barrel's drain line is always open
                         and drains continuously. This parameter is ignored for other types of
                         LIDs.
+                    Hopen (): The height of water (in inches or mm) in the drain's Storage Layer that causes
+                        the drain to automatically open. Use 0 to disable this feature.
+                    Hclose (): The height of water (in inches or mm) in the drain's Storage Layer that causes
+                        the drain to automatically close. Use 0 to disable this feature.
+                    Qcurve (): The name of an optional Control Curve that adjusts the computed drain flow as
+                        a function of the head of water above the drain. Leave blank if not applicable.
                 """
                 self.Coeff = float(Coeff)
                 self.Expon = float(Expon)
                 self.Offset = float(Offset)
                 self.Delay = Delay  # acc. to documentation  / for Rain Barrels only
-                # self.open_level = open_level  # to in documentation
-                # self.close_level = close_level  # to in documentation
+                self.open_level = open_level  # not in documentation of 5.1 but in 5.2
+                self.close_level = close_level  # not in documentation of 5.1 but in 5.2
+                self.Qcurve = Qcurve
 
         class Drainmat(BaseSectionObject):
             _LABEL = 'DRAINMAT'
@@ -273,79 +308,93 @@ class LIDControl(BaseSectionObject):
                 self.Vratio = float(Vratio)
                 self.Rough = float(Rough)
 
+        class Removals(BaseSectionObject):
+            _LABEL = 'REMOVALS'
+
+            def __init__(self, *pollutant_removal_rate):
+                """
+                Args:
+                    Pollut (): name of a pollutant
+                    Rmvl (): the percent removal the LID achieves for the pollutant (several pollutant
+                        removals can be placed on the same line or specified in separate REMOVALS
+                        lines).
+                """
+                self.pollutant_removal_rate = list(pollutant_removal_rate)
+
         SURFACE = Surface._LABEL
         SOIL = Soil._LABEL
         PAVEMENT = Pavement._LABEL
         STORAGE = Storage._LABEL
         DRAIN = Drain._LABEL
         DRAINMAT = Drainmat._LABEL
+        REMOVALS = Removals._LABEL
 
-        _possible = [SURFACE, SOIL, PAVEMENT, STORAGE, DRAIN, DRAINMAT]
+        _possible = [SURFACE, SOIL, PAVEMENT, STORAGE, DRAIN, DRAINMAT, REMOVALS]
 
-        _dict = {x._LABEL: x for x in (Surface, Soil, Pavement, Storage, Drain, Drainmat)}
+        _dict = {x._LABEL: x for x in (Surface, Soil, Pavement, Storage, Drain, Drainmat, Removals)}
 
     def to_inp_line(self):
-        s = '{} {}\n'.format(self.Name, self.lid_kind)
+        s = '{} {}\n'.format(self.name, self.lid_kind)
         for layer, l in self.layer_dict.items():
-            s += '{} {:<8} '.format(self.Name, layer) + l.to_inp_line() + '\n'
+            s += '{} {:<8} '.format(self.name, layer) + l.to_inp_line() + '\n'
         return s
 
 
 class LIDUsage(BaseSectionObject):
     """
-    Section: [**LID_USAGE**]
+    Assignment of LID controls to subcatchments.
+
+    Section:
+        [LID_USAGE]
 
     Purpose:
         Deploys LID controls within specific subcatchment areas.
 
-    Formats:
-        ::
+    Remarks:
+        If :attr:`LIDUsage.route_to_pervious` is set to 1 and :attr:`LIDUsage.drain_to` set to some other outlet,
+        then only the excess surface flow from the LID unit will be routed back to the subcatchment’s pervious
+        area while the underdrain flow will be sent to :attr:`LIDUsage.drain_to`.
 
-            Subcat LID Number Area Width InitSat FromImp ToPerv (RptFile DrainTo)
+        More than one type of LID process can be deployed within a subcatchment as long
+        as their total area does not exceed that of the subcatchment and the total percent
+        impervious area treated does not exceed 100.
 
-    Args:
-        Subcat (str): name of the subcatchment using the LID process.
-        LID (str): name of an LID process defined in the [LID_CONTROLS] section.
-        Number (int): number of replicate LID units deployed.
-        Area (float): area of each replicate unit (ft^2 or m^2 ).
-        Width (float): width of the outflow face of each identical LID unit (in ft or m). This
+    Attributes:
+        subcatchment (str): Name of the subcatchment using the LID process.
+        lid (str): Name of an LID process defined in the [LID_CONTROLS] section.
+        n_replicate (int): Number of replicate LID units deployed.
+        area (float): Area of each replicate unit (ft^2 or m^2 ).
+        width (float): Width of the outflow face of each identical LID unit (in ft or m). This
             parameter applies to roofs, pavement, trenches, and swales that use
             overland flow to convey surface runoff off of the unit. It can be set to 0 for
             other LID processes, such as bio-retention cells, rain gardens, and rain
             barrels that simply spill any excess captured runoff over their berms.
-        InitSat (float): for bio-retention cells, rain gardens, and green roofs this is the degree to
+        saturation_init (float): For bio-retention cells, rain gardens, and green roofs this is the degree to
             which the unit's soil is initially filled with water (0 % saturation
             corresponds to the wilting point moisture content, 100 % saturation has
             the moisture content equal to the porosity). The storage zone beneath
             the soil zone of the cell is assumed to be completely dry. For other types
             of LIDs it corresponds to the degree to which their storage zone is
             initially filled with water
-        FromImp (float): percent of the impervious portion of the subcatchment’s non-LID area
+        impervious_portion (float): Percent of the impervious portion of the subcatchment’s non-LID area
             whose runoff is treated by the LID practice. (E.g., if rain barrels are used
             to capture roof runoff and roofs represent 60% of the impervious area,
             then the impervious area treated is 60%). If the LID unit treats only direct
             rainfall, such as with a green roof, then this value should be 0. If the LID
             takes up the entire subcatchment then this field is ignored.
-        ToPerv (int): a value of 1 indicates that the surface and drain flow from the LID unit
+        route_to_pervious (int): A value of 1 indicates that the surface and drain flow from the LID unit
             should be routed back onto the pervious area of the subcatchment that
             contains it. This would be a common choice to make for rain barrels,
             rooftop disconnection, and possibly green roofs. The default value is 0.
-        RptFile (str): optional name of a file to which detailed time series results for the LID
+        fn_lid_report (str): Optional name of a file to which detailed time series results for the LID
             will be written. Enclose the name in double quotes if it contains spaces
             and include the full path if it is different than the SWMM input file path.
             Use ‘*’ if not applicable and an entry for DrainTo follows
-        DrainTo (str): optional name of subcatchment or node that receives flow from the unit’s
+        drain_to (str): Optional name of subcatchment or node that receives flow from the unit’s
             drain line, if different from the outlet of the subcatchment that the LID is
             placed in.
-
-    Remarks:
-        If ``ToPerv`` is set to 1 and ``DrainTo`` set to some other outlet, then only the excess
-        surface flow from the LID unit will be routed back to the subcatchment’s pervious
-        area while the underdrain flow will be sent to ``DrainTo``.
-
-        More than one type of LID process can be deployed within a subcatchment as long
-        as their total area does not exceed that of the subcatchment and the total percent
-        impervious area treated does not exceed 100.
+        from_pervious (float): optional percent of the pervious portion of the subcatchment’s non-LID area
+            whose runoff is treated by the LID practice. The default value is 0.
 
     Examples:
         ::
@@ -361,17 +410,58 @@ class LIDUsage(BaseSectionObject):
             ;swale 200 ft long by 50 ft wide.
             S2 Swale 1 10000 50 0 0 0 “swale.rpt”
     """
-    _identifier = [IDENTIFIERS.Subcatch, 'LID']
+    _identifier = (IDENTIFIERS.subcatchment, 'lid')
     _section_label = LID_USAGE
 
-    def __init__(self, Subcatch, LID, Number, Area, Width, InitSat, FromImp, ToPerv, RptFile=NaN, DrainTo=NaN):
-        self.Subcatch = str(Subcatch)
-        self.LID = str(LID)
-        self.Number = Number
-        self.Area = float(Area)
-        self.Width = float(Width)
-        self.InitSat = float(InitSat)
-        self.FromImp = float(FromImp)
-        self.ToPerv = int(ToPerv)
-        self.RptFile = RptFile
-        self.DrainTo = DrainTo
+    def __init__(self, subcatchment, lid, n_replicate, area, width, saturation_init, impervious_portion, route_to_pervious=0, fn_lid_report=NaN, drain_to=NaN, from_pervious=NaN):
+        """
+        Assignment of LID controls to subcatchments.
+
+        Args:
+            subcatchment (str): Name of the subcatchment using the LID process.
+            lid (str): Name of an LID process defined in the [LID_CONTROLS] section.
+            n_replicate (int): Number of replicate LID units deployed.
+            area (float): Area of each replicate unit (ft^2 or m^2 ).
+            width (float): Width of the outflow face of each identical LID unit (in ft or m). This
+                parameter applies to roofs, pavement, trenches, and swales that use
+                overland flow to convey surface runoff off of the unit. It can be set to 0 for
+                other LID processes, such as bio-retention cells, rain gardens, and rain
+                barrels that simply spill any excess captured runoff over their berms.
+            saturation_init (float): For bio-retention cells, rain gardens, and green roofs this is the degree to
+                which the unit's soil is initially filled with water (0 % saturation
+                corresponds to the wilting point moisture content, 100 % saturation has
+                the moisture content equal to the porosity). The storage zone beneath
+                the soil zone of the cell is assumed to be completely dry. For other types
+                of LIDs it corresponds to the degree to which their storage zone is
+                initially filled with water
+            impervious_portion (float): Percent of the impervious portion of the subcatchment’s non-LID area
+                whose runoff is treated by the LID practice. (E.g., if rain barrels are used
+                to capture roof runoff and roofs represent 60% of the impervious area,
+                then the impervious area treated is 60%). If the LID unit treats only direct
+                rainfall, such as with a green roof, then this value should be 0. If the LID
+                takes up the entire subcatchment then this field is ignored.
+            route_to_pervious (int): A value of 1 indicates that the surface and drain flow from the LID unit
+                should be routed back onto the pervious area of the subcatchment that
+                contains it. This would be a common choice to make for rain barrels,
+                rooftop disconnection, and possibly green roofs. The default value is 0.
+            fn_lid_report (str): Optional name of a file to which detailed time series results for the LID
+                will be written. Enclose the name in double quotes if it contains spaces
+                and include the full path if it is different than the SWMM input file path.
+                Use ‘*’ if not applicable and an entry for DrainTo follows
+            drain_to (str): Optional name of subcatchment or node that receives flow from the unit’s
+                drain line, if different from the outlet of the subcatchment that the LID is
+                placed in.
+            from_pervious (float): optional percent of the pervious portion of the subcatchment’s non-LID area
+                whose runoff is treated by the LID practice. The default value is 0.
+        """
+        self.subcatchment = str(subcatchment)
+        self.lid = str(lid)
+        self.n_replicate = n_replicate
+        self.area = float(area)
+        self.width = float(width)
+        self.saturation_init = float(saturation_init)
+        self.impervious_portion = float(impervious_portion)
+        self.route_to_pervious = int(route_to_pervious)
+        self.fn_lid_report = fn_lid_report
+        self.drain_to = drain_to
+        self.from_pervious = from_pervious
