@@ -21,7 +21,15 @@ class SubCatchment(BaseSectionObject):
         units which generate runoff from rainfall.
 
     Attributes:
-
+        name (str): Name assigned to subcatchment.
+        rain_gage (str): Name of rain gage in [``RAINGAGES``] section (:class:`~swmm_api.input_file.sections.RainGage`) assigned to subcatchment.
+        outlet (str): Name of node or subcatchment that receives runoff from subcatchment.
+        area (float): Area of subcatchment (acres or hectares).
+        imperviousness (float): Percent imperviousness of subcatchment.
+        width (float): Characteristic width of subcatchment (ft or meters).
+        slope (float): Subcatchment slope (percent).
+        curb_length (float): Total curb length (any length units). Use 0 if not applicable.
+        snow_pack (str): Optional name of snow pack object (from [``SNOWPACKS``] section) that characterizes snow accumulation and melting over the subcatchment.
     """
     _identifier = IDENTIFIERS.name
     _section_label = SUBCATCHMENTS
@@ -32,8 +40,7 @@ class SubCatchment(BaseSectionObject):
 
         Args:
             name (str): Name assigned to subcatchment.
-
-            rain_gage (str): Name of rain gage in [``RAINGAGES`` ` ``RAINGAGES`` :class:`RainGage` `_ ] section assigned to subcatchment.
+            rain_gage (str): Name of rain gage in [``RAINGAGES``] section (:class:`~swmm_api.input_file.sections.RainGage`) assigned to subcatchment.
             outlet (str): Name of node or subcatchment that receives runoff from subcatchment.
             area (float): Area of subcatchment (acres or hectares).
             imperviousness (float): Percent imperviousness of subcatchment.
@@ -441,7 +448,7 @@ class Polygon(BaseSectionObject):
         has_interiors = data.interiors.apply(len) > 0
         if has_interiors.any():
             warnings.warn('Converting GeoSeries with Interiors(Holes) to POLYGON inp-section will ignore this interiors!')
-        # s.add_multiple(cls(i, [xy[0:2] for xy in list(p.coords)]) for i, p in data.exterior.iteritems())
+        # s.add_multiple(cls(i, [xy[0:2] for xy in list(p.coords)]) for i, p in data.exterior.items())
         s.add_multiple(cls.from_shapely(i, p) for i, p in data.items())
         return s
 
