@@ -8,16 +8,14 @@
 # Python Version >= 3.7
 
 import copy
-from os import remove
 
 import datetime
-import struct
 from io import SEEK_END, SEEK_SET
 from tqdm.auto import tqdm
 from warnings import warn
 
 from .definitions import OBJECTS, VARIABLES
-from .._read_bin import BinaryReader
+from .._io_helpers._read_bin import BinaryReader
 
 VARIABLES_DICT = {
     OBJECTS.SUBCATCHMENT: VARIABLES.SUBCATCHMENT.LIST_,
@@ -48,11 +46,49 @@ class SwmmOutExtractWarning(UserWarning):
 
 
 class SwmmOutExtract(BinaryReader):
-    """The class that handles all extraction of data from the out file."""
+    """
+    The class that handles all extraction of data from the out file.
 
-    def __init__(self, filename):
-        super().__init__(filename)
+    Attributes:
+        flow_unit (str): Flow unit. One of ['CMS', 'LPS', 'MLD', 'CFS', 'GPM', 'MGD']
+        labels (dict[str, list]): dictionary of the object labels as list (value) for each object type
+            (keys are: ``'link'``, ``'node'``, ``'subcatchment'``)
+        model_properties (dict[str, [dict[str, list]]]): property values for the subcatchments, nodes and links. 
+            The Properties for the objects are.
+        
+                    - ``subcatchment``
+                      - [area]
+                    - ``node``
+                      - [type, invert, max. depth]
+                    - ``link``
+                      - type, 
+                      - offsets
+                        - ht. above start node invert (ft), 
+                        - ht. above end node invert (ft), 
+                      - max. depth,
+                      - length
+        
+        n_periods (int): number of periods (=index-values)
+        pollutant_units (dict[str, str]): Units per pollutant.
+        _pos_start_output (int): Start position of the data.
+        report_interval (datetime.timedelta): Intervall of the index.
+        start_date (datetime.datetime): Start date of the data.
+        swmm_version (str): SWMM Version
+        variables (dict[str, list]): variables per object-type inclusive the pollutants.
+        fp (file-like): Stream of the open file.
+        filename (str): Path to the .out-file.
 
+    Args:
+        filename (str): Path to the .out-file.
+        encoding (str): Encoding of the text in the binary-file (None -> auto-detect encoding ... takes a few seconds | '' -> use default = 'utf-8')
+    """
+    filename: str
+
+    def __init__(self, filename, skip_init=False, encoding=''):
+        super().__init__(filename, encoding)
+
+        if skip_init:
+            return
         # ____
         self.fp.seek(-6 * _RECORDSIZE, SEEK_END)
         (
@@ -68,6 +104,7 @@ class SwmmOutExtract(BinaryReader):
         self.fp.seek(0, SEEK_SET)
         magic_num_start = self._next()
 
+        self.run_failed = False
         # ____
         # check errors
         if magic_num_start != _MAGIC_NUMBER:
@@ -77,9 +114,11 @@ class SwmmOutExtract(BinaryReader):
             warn('Ending magic number incorrect.', SwmmOutExtractWarning)
             # raise SwmmExtractValueError('Ending magic number incorrect.')
             _n_periods = 0
+            self.run_failed = True
         elif error_code != 0:
             warn(f'Error code "{error_code}" in output file indicates a problem with the run.', SwmmOutExtractWarning)
             # raise SwmmExtractValueError(f'Error code "{error_code}" in output file indicates a problem with the run.')
+            self.run_failed = True
 
         # ---
         # read additional parameters from start of file
@@ -94,7 +133,7 @@ class SwmmOutExtract(BinaryReader):
         # ____
         # Read in the names
         # get the dictionary of the object labels for each object type (link, node, subcatchment)
-        self.labels = dict()
+        self.labels = {}
         for kind, n in zip(OBJECTS.LIST_, [n_subcatch, n_nodes, n_links, n_pollutants, 0]):
             self.labels[kind] = [self._next(n=self._next(), dtype='s') for _ in range(n)]
 
@@ -126,12 +165,12 @@ class SwmmOutExtract(BinaryReader):
         #     type, invert, & max. depth
         #   link
         #     type, offsets [ht. above start node invert (ft), ht. above end node invert (ft)], max. depth, & length
-        self.model_properties = dict()
+        self.model_properties = {}
         for kind in [OBJECTS.SUBCATCHMENT, OBJECTS.NODE, OBJECTS.LINK]:
-            self.model_properties[kind] = dict()
+            self.model_properties[kind] = {}
             # ------
             # read the property labels per object type
-            property_labels = list()
+            property_labels = []
             for i in list(self._next(self._next(), flat=False)):
                 property_label = _PROPERTY_LABELS[i]
                 if property_label in property_labels:
@@ -140,7 +179,7 @@ class SwmmOutExtract(BinaryReader):
             # ------
             # read the values per object and per property
             for label in self.labels[kind]:
-                self.model_properties[kind][label] = dict()
+                self.model_properties[kind][label] = {}
                 for property_label in property_labels:
                     value = self._next(dtype={'type': 'i'}.get(property_label, 'f'))
                     if property_label == 'type':
@@ -159,18 +198,18 @@ class SwmmOutExtract(BinaryReader):
         self.report_interval = datetime.timedelta(seconds=self._next())
 
         # ____
-        self._bytes_per_period = None
+        self._bytes_per_period = self._infer_bytes_per_period()
 
         # ____
         # print(self.fp.tell(), _pos_start_output)
         # assert _pos_start_output == self.fp.tell()
         # if _pos_start_output == 0:
         # Out File not complete!
-        self.pos_start_output = self.fp.tell()
+        self._pos_start_output = self.fp.tell()
 
         self.n_periods = _n_periods
         if _n_periods == 0:
-            self.infer_n_periods()
+            self._infer_n_periods()
             warn('Infer time periods of the output file due to an corrupt SWMM .out-file.', SwmmOutExtractWarning)
 
         if self.n_periods == 0:
@@ -180,24 +219,21 @@ class SwmmOutExtract(BinaryReader):
     def __repr__(self):
         return f'SwmmOutExtract(file="{self.filename}")'
 
-    @property
-    def bytes_per_period(self):
+    def _infer_bytes_per_period(self):
         """
         Calculate the bytes for each time period when reading the computed results
 
         Returns:
             int: bytes per period
         """
-        if self._bytes_per_period is None:
-            self._bytes_per_period = 2  # for the datetime
-            for obj in [OBJECTS.SUBCATCHMENT, OBJECTS.NODE, OBJECTS.LINK]:
-                self._bytes_per_period += len(self.variables[obj]) * len(self.labels[obj])
-            self._bytes_per_period += len(self.variables[OBJECTS.SYSTEM])
-            self._bytes_per_period *= _RECORDSIZE
+        _bytes_per_period = 2  # for the datetime
+        for obj in [OBJECTS.SUBCATCHMENT, OBJECTS.NODE, OBJECTS.LINK]:
+            _bytes_per_period += len(self.variables[obj]) * len(self.labels[obj])
+        _bytes_per_period += len(self.variables[OBJECTS.SYSTEM])
+        _bytes_per_period *= _RECORDSIZE
+        return _bytes_per_period
 
-        return self._bytes_per_period
-
-    def get_selective_results(self, columns):
+    def _get_selective_results(self, columns, processes=1):
         """
         get results of selective columns in .out-file
 
@@ -218,11 +254,11 @@ class SwmmOutExtract(BinaryReader):
         n_nodes = len(self.labels[OBJECTS.NODE])
         n_links = len(self.labels[OBJECTS.LINK])
 
-        offset_list = list()
-        values = dict()
+        offset_list = []
+        values = {}
 
         for kind, label, variable in columns:
-            values['/'.join([kind, label, variable])] = list()
+            values['/'.join([kind, label, variable])] = []
 
             index_kind = OBJECTS.LIST_.index(kind)
             index_variable = self.variables[kind].index(variable)
@@ -246,22 +282,48 @@ class SwmmOutExtract(BinaryReader):
         # iter_label_offset = tuple(zip(cols_sorted, offset_sorted))
         iter_label_offset = tuple(zip(values.keys(), offset_list))
 
-        for period_offset in tqdm(range(self.pos_start_output,  # start
-                                        self.pos_start_output + self.n_periods * self.bytes_per_period,  # stop
-                                        self.bytes_per_period),
-                                  desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)})'):  # step
-            # period_offset = self.pos_start_output + period * self.bytes_per_period
+        variable = range(self._pos_start_output,  # start
+                         self._pos_start_output + self.n_periods * self._bytes_per_period,  # stop
+                         self._bytes_per_period)
+
+        def func(out, period_offset):
             for label, offset in iter_label_offset:
-                self._set_position(offset + period_offset)
-                values[label].append(self._next_float())
+                out._set_position(offset + period_offset)
+                values[label].append(out._next_float())
+
+        # for period_offset in tqdm(range(self._pos_start_output,  # start
+        #                                 self._pos_start_output + self.n_periods * self._bytes_per_period,  # stop
+        #                                 self._bytes_per_period),
+        #                           desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)})'):  # step
+        #     # period_offset = self.pos_start_output + period * self.bytes_per_period
+        #     for label, offset in iter_label_offset:
+        #         self._set_position(offset + period_offset)
+        #         values[label].append(self._next_float())
+
+        # -------------
+        from functools import partial
+        from itertools import cycle
+
+        if processes == 1:
+            for period_offset in tqdm(variable, desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)})'):
+                func(self, period_offset)
+
+        else:
+            from multiprocessing.dummy import Pool
+
+            files = [self.copy() for _ in range(processes)]
+            pool = Pool(processes)
+            for _ in tqdm(pool.starmap(partial(func), zip(cycle(files), variable)), total=len(variable), desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)}, processes={processes})'):
+                pass
+        # -------------
 
         return values
 
-    def infer_n_periods(self):
+    def _infer_n_periods(self):
         not_done = True
         period = 0
         while not_done:
-            self.fp.seek(self.pos_start_output + period * self.bytes_per_period, SEEK_SET)
+            self.fp.seek(self._pos_start_output + period * self._bytes_per_period, SEEK_SET)
             try:
                 dt = self._next(dtype='d')
                 # print(dt)
@@ -271,3 +333,8 @@ class SwmmOutExtract(BinaryReader):
                 not_done = False
 
         self.n_periods = period - 1
+
+    def copy(self):
+        new = type(self)(self.filename, skip_init=True)
+        new.__dict__ = vars(self)
+        return new
