@@ -6,6 +6,7 @@ __version__ = "0.1"
 __license__ = "MIT"
 
 import datetime
+import warnings
 from itertools import product
 from numpy import dtype, fromfile, frombuffer
 from pandas import date_range, DataFrame, MultiIndex
@@ -17,6 +18,10 @@ from .definitions import OBJECTS, VARIABLES
 from . import parquet_helpers as parquet
 
 
+class SwmmOutputWarning(UserWarning):
+    pass
+
+
 class SwmmOutput(SwmmOutExtract):
     """
     Read the SWMM Output file (xxx.out).
@@ -56,22 +61,26 @@ class SwmmOutput(SwmmOutExtract):
         swmm_version (str): SWMM Version
         variables (dict[str, list]): variables per object-type inclusive the pollutants.
         fp (file-like): Stream of the open file.
-        filename (str): Path to the .out-file.
+        filename (str): Path to the output-file (.out).
 
     .. _swmmtoolbox: https://github.com/timcera/swmmtoolbox
     """
-    def __init__(self, filename):
+    def __init__(self, filename, skip_init=False, encoding=''):
         """
         Read the SWMM Output file (xxx.out).
 
         Args:
-            filename(str): Path to the .out-file.
+            filename(str): Path to the output-file (.out).
+            encoding (str): Encoding of the text in the binary-file (None -> auto-detect encoding ... takes a few seconds | '' -> use default = 'utf-8')
         """
-        SwmmOutExtract.__init__(self, filename)
+        SwmmOutExtract.__init__(self, filename, skip_init=skip_init, encoding=encoding)
 
         self._frame = None
         self._data = None
 
+        if skip_init:
+            return
+
         # the main datetime index for the results
         try:
             self.index = date_range(self.start_date + self.report_interval,
@@ -150,7 +159,7 @@ class SwmmOutput(SwmmOutExtract):
             del self._frame['datetime']
         return self._frame
 
-    def get_part(self, kind=None, label=None, variable=None, slim=False):
+    def get_part(self, kind=None, label=None, variable=None, slim=False, processes=1):
         """
         Get specific columns of the data.
 
@@ -210,7 +219,7 @@ class SwmmOutput(SwmmOutExtract):
         """
         columns = self._filter_part_columns(kind, label, variable)
         if slim:
-            values = self._get_selective_results(columns)
+            values = self._get_selective_results(columns, processes=processes)
         else:
             values = self.to_numpy()[list(map('/'.join, columns))]
 
@@ -229,20 +238,28 @@ class SwmmOutput(SwmmOutExtract):
             list: filtered list of tuple(kind, label, variable)
         """
 
-        def _filter(i, possibilities):
+        def _filter(i, possibilities, error_label):
             if i is None:
                 return possibilities
             elif isinstance(i, str):
                 if i in possibilities:
                     return [i]
                 else:
+                    warnings.warn(f'Did not found {error_label} "{i}" in output-file, return empty data. Possibilities: {possibilities}.', SwmmOutputWarning)
                     return []
             elif isinstance(i, list):
-                return [j for j in i if j in possibilities]
+                # return [j for j in i if j in possibilities]
+                l = []
+                for j in i:
+                    if j in possibilities:
+                        l.append(j)
+                    else:
+                        warnings.warn(f'Did not found {error_label} "{j}" in output-file, skipping request. Possibilities: {possibilities}', SwmmOutputWarning)
+                return l
 
         columns = []
-        for k in _filter(kind, OBJECTS.LIST_):
-            columns += list(product([k], _filter(label, self.labels[k]), _filter(variable, self.variables[k])))
+        for k in _filter(kind, OBJECTS.LIST_, 'object kind'):
+            columns += list(product([k], _filter(label, self.labels[k], f'{k} label'), _filter(variable, self.variables[k], f'{k} variable')))
         return columns
 
     def _to_pandas(self, data, drop_useless=False):
@@ -328,20 +345,22 @@ class SwmmOutput(SwmmOutExtract):
     #     )
 
 
-def read_out_file(filename):
-    """
-    Read the SWMM Output file (xxx.out).
+read_out_file = SwmmOutput
 
-    Args:
-        filename (str): filename of the output file
-
-    Returns:
-        SwmmOutput: output file object
-
-    See Also:
-        :meth:`SwmmOutput.__init__` : Equal functionality.
-    """
-    return SwmmOutput(filename)
+# def read_out_file(filename):
+#     """
+#     Read the SWMM Output file (xxx.out).
+#
+#     Args:
+#         filename (str): filename of the output file
+#
+#     Returns:
+#         SwmmOutput: output file object
+#
+#     See Also:
+#         :meth:`SwmmOutput.__init__` : Equal functionality.
+#     """
+#     return SwmmOutput(filename)
 
 
 def out2frame(filename):
